@@ -17,10 +17,50 @@ clean_tool_cache() {
 }
 # npm/pnpm/yarn/bun caches.
 clean_dev_npm() {
+    local npm_default_cache="$HOME/.npm"
+    local npm_cache_path="$npm_default_cache"
+
     if command -v npm > /dev/null 2>&1; then
         clean_tool_cache "npm cache" npm cache clean --force
+
+        start_section_spinner "Checking npm cache path..."
+        npm_cache_path=$(run_with_timeout 2 npm config get cache 2> /dev/null) || npm_cache_path=""
+        stop_section_spinner
+
+        if [[ -z "$npm_cache_path" || "$npm_cache_path" != /* ]]; then
+            npm_cache_path="$npm_default_cache"
+        fi
+
         note_activity
     fi
+
+    # These residual directories are not removed by `npm cache clean --force`
+    local -a npm_residual_dirs=("_cacache" "_npx" "_logs" "_prebuilds")
+    local -a npm_descriptions=("npm cache directory" "npm npx cache" "npm logs" "npm prebuilds")
+
+    # Clean default npm cache path
+    local i
+    for i in "${!npm_residual_dirs[@]}"; do
+        safe_clean "$npm_default_cache/${npm_residual_dirs[$i]}"/* "${npm_descriptions[$i]}"
+    done
+
+    # Normalize paths for comparison (remove trailing slash + resolve symlinked dirs)
+    local npm_cache_path_normalized="${npm_cache_path%/}"
+    local npm_default_cache_normalized="${npm_default_cache%/}"
+    if [[ -d "$npm_cache_path_normalized" ]]; then
+        npm_cache_path_normalized=$(cd "$npm_cache_path_normalized" 2> /dev/null && pwd -P) || npm_cache_path_normalized="${npm_cache_path%/}"
+    fi
+    if [[ -d "$npm_default_cache_normalized" ]]; then
+        npm_default_cache_normalized=$(cd "$npm_default_cache_normalized" 2> /dev/null && pwd -P) || npm_default_cache_normalized="${npm_default_cache%/}"
+    fi
+
+    # Clean custom npm cache path (if different from default)
+    if [[ "$npm_cache_path_normalized" != "$npm_default_cache_normalized" ]]; then
+        for i in "${!npm_residual_dirs[@]}"; do
+            safe_clean "$npm_cache_path/${npm_residual_dirs[$i]}"/* "${npm_descriptions[$i]} (custom path)"
+        done
+    fi
+
     # Clean pnpm store cache
     local pnpm_default_store=~/Library/pnpm/store
     # Check if pnpm is actually usable (not just Corepack shim)
@@ -326,7 +366,10 @@ clean_xcode_simulator_runtime_volumes() {
         [[ -n "$sorted" ]] && sorted_candidates+=("$sorted")
     done < <(printf '%s\n' "${candidates[@]}" | LC_ALL=C sort)
 
-    echo -e "  ${GRAY}${ICON_LIST}${NC} Xcode runtime volumes · scanning ${#sorted_candidates[@]} entries"
+    # Only show scanning message in debug mode; spinner provides visual feedback otherwise
+    if [[ "${MO_DEBUG:-0}" == "1" ]]; then
+        echo -e "  ${GRAY}${ICON_LIST}${NC} Xcode runtime volumes · scanning ${#sorted_candidates[@]} entries"
+    fi
     local runtime_scan_spinner=false
     if [[ -t 1 ]]; then
         start_section_spinner "Scanning Xcode runtime volumes..."
@@ -434,24 +477,20 @@ clean_xcode_simulator_runtime_volumes() {
     fi
 
     if [[ ${#selected_paths[@]} -eq 0 ]]; then
-        echo -e "  ${GREEN}${ICON_SUCCESS}${NC} Xcode runtime volumes · nothing to clean"
+        echo -e "  ${GREEN}${ICON_SUCCESS}${NC} Xcode runtime volumes · already clean"
         note_activity
         return 0
     fi
 
     if ! has_sudo_session; then
         if ! ensure_sudo_session "Cleaning Xcode runtime volumes requires admin access"; then
-            echo -e "  ${GRAY}${ICON_WARNING}${NC} Xcode runtime volumes cleanup skipped (sudo denied)"
+            echo -e "  ${YELLOW}${ICON_WARNING}${NC} Xcode runtime volumes · skipped (sudo denied)"
             note_activity
             return 0
         fi
     fi
 
-    echo -e "  ${GREEN}${ICON_SUCCESS}${NC} Xcode runtime volumes · cleaning ${#selected_paths[@]} unused"
-    if [[ $skipped_protected -gt 0 ]]; then
-        echo -e "  ${GRAY}${ICON_WARNING}${NC} Xcode runtime volumes · skipped ${skipped_protected} protected items"
-    fi
-
+    # Perform cleanup and report final result in one line
     local removed_count=0
     local removed_size_kb=0
     local selected_path
@@ -464,13 +503,22 @@ clean_xcode_simulator_runtime_volumes() {
         fi
     done
 
+    # Unified output: report result, not intermediate steps
     if [[ $removed_count -gt 0 ]]; then
         local removed_human
         removed_human=$(bytes_to_human "$((removed_size_kb * 1024))")
-        echo -e "  ${GREEN}${ICON_SUCCESS}${NC} Xcode runtime volumes · removed ${removed_count}, ${removed_human}"
+        if [[ $skipped_protected -gt 0 ]]; then
+            echo -e "  ${GREEN}${ICON_SUCCESS}${NC} Xcode runtime volumes · removed ${removed_count} (${removed_human}), skipped ${skipped_protected} protected"
+        else
+            echo -e "  ${GREEN}${ICON_SUCCESS}${NC} Xcode runtime volumes · removed ${removed_count} (${removed_human})"
+        fi
         note_activity
     else
-        echo -e "  ${GRAY}${ICON_WARNING}${NC} Xcode runtime volumes · no items removed"
+        if [[ $skipped_protected -gt 0 ]]; then
+            echo -e "  ${YELLOW}${ICON_WARNING}${NC} Xcode runtime volumes · skipped ${skipped_protected} protected, none removed"
+        else
+            echo -e "  ${GREEN}${ICON_SUCCESS}${NC} Xcode runtime volumes · already clean"
+        fi
         note_activity
     fi
 }
@@ -555,12 +603,28 @@ clean_dev_mobile() {
     safe_clean ~/.android/cache/* "Android SDK cache"
     safe_clean ~/Library/Developer/Xcode/UserData/IB\ Support/* "Xcode Interface Builder cache"
     safe_clean ~/.cache/swift-package-manager/* "Swift package manager cache"
+    # Expo/React Native caches (preserve state.json which contains auth tokens).
+    safe_clean ~/.expo/expo-go/* "Expo Go cache"
+    safe_clean ~/.expo/android-apk-cache/* "Expo Android APK cache"
+    safe_clean ~/.expo/ios-simulator-app-cache/* "Expo iOS simulator app cache"
+    safe_clean ~/.expo/native-modules-cache/* "Expo native modules cache"
+    safe_clean ~/.expo/schema-cache/* "Expo schema cache"
+    safe_clean ~/.expo/template-cache/* "Expo template cache"
+    safe_clean ~/.expo/versions-cache/* "Expo versions cache"
 }
 # JVM ecosystem caches.
-# Gradle excluded (default whitelist, like Maven). Remove via: mo clean --whitelist
+# Gradle: Respects whitelist, cleaned when not protected via: mo clean --whitelist
 clean_dev_jvm() {
+    # Source Maven cleanup module (requires bash for BASH_SOURCE)
+    # shellcheck disable=SC1091
+    source "$(dirname "${BASH_SOURCE[0]}")/maven.sh" 2> /dev/null || true
+    if declare -f clean_maven_repository > /dev/null 2>&1; then
+        clean_maven_repository
+    fi
     safe_clean ~/.sbt/* "SBT cache"
     safe_clean ~/.ivy2/cache/* "Ivy cache"
+    safe_clean ~/.gradle/caches/* "Gradle cache"
+    safe_clean ~/.gradle/daemon/* "Gradle daemon"
 }
 # JetBrains Toolbox old IDE versions (keep current + recent backup).
 clean_dev_jetbrains_toolbox() {
@@ -782,6 +846,8 @@ clean_dev_editors() {
 # Main developer tools cleanup sequence.
 clean_developer_tools() {
     stop_section_spinner
+
+    # CLI tools and languages
     clean_sqlite_temp_files
     clean_dev_npm
     clean_dev_python
@@ -806,9 +872,13 @@ clean_developer_tools() {
     clean_dev_elixir
     clean_dev_haskell
     clean_dev_ocaml
-    clean_dev_editors
+
+    # GUI developer applications
+    clean_xcode_tools
+    clean_code_editors
+
+    # Homebrew
     safe_clean ~/Library/Caches/Homebrew/* "Homebrew cache"
-    # Clean Homebrew locks without repeated sudo prompts.
     local brew_lock_dirs=(
         "/opt/homebrew/var/homebrew/locks"
         "/usr/local/var/homebrew/locks"
