@@ -89,13 +89,18 @@ paginated_multi_select() {
     local top_index=0
     local sort_mode="${MOLE_MENU_SORT_MODE:-${MOLE_MENU_SORT_DEFAULT:-date}}" # date|name|size
     local sort_reverse="${MOLE_MENU_SORT_REVERSE:-false}"
+    local filter_text="" # Filter keyword
+    local filter_text_lower=""
 
     # Metadata (optional)
     # epochs[i]   -> last_used_epoch (numeric) for item i
     # sizekb[i]   -> size in KB (numeric) for item i
+    # filter_names[i] -> name for filtering (if not set, use items[i])
     local -a epochs=()
     local -a sizekb=()
+    local -a filter_names=()
     local has_metadata="false"
+    local has_filter_names="false"
     if [[ -n "${MOLE_MENU_META_EPOCHS:-}" ]]; then
         while IFS= read -r v; do epochs+=("${v:-0}"); done < <(_pm_parse_csv_to_array "$MOLE_MENU_META_EPOCHS")
         has_metadata="true"
@@ -104,6 +109,10 @@ paginated_multi_select() {
         while IFS= read -r v; do sizekb+=("${v:-0}"); done < <(_pm_parse_csv_to_array "$MOLE_MENU_META_SIZEKB")
         has_metadata="true"
     fi
+    if [[ -n "${MOLE_MENU_FILTER_NAMES:-}" ]]; then
+        while IFS= read -r v; do filter_names+=("$v"); done <<< "$MOLE_MENU_FILTER_NAMES"
+        has_filter_names="true"
+    fi
 
     # If no metadata, force name sorting and disable sorting controls
     if [[ "$has_metadata" == "false" && "$sort_mode" != "name" ]]; then
@@ -113,10 +122,20 @@ paginated_multi_select() {
     # Index mappings
     local -a orig_indices=()
     local -a view_indices=()
+    local -a filter_targets_lower=()
     local i
     for ((i = 0; i < total_items; i++)); do
         orig_indices[i]=$i
         view_indices[i]=$i
+        local filter_target
+        if [[ $has_filter_names == true && -n "${filter_names[i]:-}" ]]; then
+            filter_target="${filter_names[i]}"
+        else
+            filter_target="${items[i]}"
+        fi
+        local filter_target_lower
+        filter_target_lower=$(printf "%s" "$filter_target" | LC_ALL=C tr '[:upper:]' '[:lower:]')
+        filter_targets_lower[i]="$filter_target_lower"
     done
 
     local -a selected=()
@@ -163,8 +182,9 @@ paginated_multi_select() {
     # Cleanup function
     cleanup() {
         trap - EXIT INT TERM
-        export MOLE_MENU_SORT_MODE="$sort_mode"
-        export MOLE_MENU_SORT_REVERSE="$sort_reverse"
+        unset MOLE_READ_KEY_FORCE_CHAR
+        export MOLE_MENU_SORT_MODE="${sort_mode:-name}"
+        export MOLE_MENU_SORT_REVERSE="${sort_reverse:-false}"
         restore_terminal
     }
 
@@ -232,55 +252,111 @@ paginated_multi_select() {
         printf "%s%s\n" "$clear_line" "$line" >&2
     }
 
-    # Rebuild the view_indices applying sort
-    rebuild_view() {
-        # Sort (skip if no metadata)
+    local sort_cache_key=""
+    local -a sorted_indices_cache=()
+    local filter_cache_key=""
+    local filter_cache_text_lower=""
+    local -a filter_cache_indices=()
+
+    ensure_sorted_indices() {
+        local requested_key="${sort_mode}:${sort_reverse}:${has_metadata}"
+        if [[ "$requested_key" == "$sort_cache_key" && ${#sorted_indices_cache[@]} -gt 0 ]]; then
+            return
+        fi
+
         if [[ "$has_metadata" == "false" ]]; then
-            # No metadata: just use original indices
-            view_indices=("${orig_indices[@]}")
-        elif [[ ${#orig_indices[@]} -eq 0 ]]; then
-            view_indices=()
+            sorted_indices_cache=("${orig_indices[@]}")
+            sort_cache_key="$requested_key"
+            return
+        fi
+
+        # Build sort key once; filtering should reuse this cached order.
+        local sort_key
+        if [[ "$sort_mode" == "date" ]]; then
+            # Date: ascending by default (oldest first)
+            sort_key="-k1,1n"
+            [[ "$sort_reverse" == "true" ]] && sort_key="-k1,1nr"
+        elif [[ "$sort_mode" == "size" ]]; then
+            # Size: descending by default (largest first)
+            sort_key="-k1,1nr"
+            [[ "$sort_reverse" == "true" ]] && sort_key="-k1,1n"
         else
-            # Build sort key
-            local sort_key
-            if [[ "$sort_mode" == "date" ]]; then
-                # Date: ascending by default (oldest first)
-                sort_key="-k1,1n"
-                [[ "$sort_reverse" == "true" ]] && sort_key="-k1,1nr"
-            elif [[ "$sort_mode" == "size" ]]; then
-                # Size: descending by default (largest first)
-                sort_key="-k1,1nr"
-                [[ "$sort_reverse" == "true" ]] && sort_key="-k1,1n"
+            # Name: ascending by default (A to Z)
+            sort_key="-k1,1f"
+            [[ "$sort_reverse" == "true" ]] && sort_key="-k1,1fr"
+        fi
+
+        local tmpfile
+        tmpfile=$(mktemp 2> /dev/null) || tmpfile=""
+        if [[ -n "$tmpfile" ]]; then
+            local k id
+            for id in "${orig_indices[@]}"; do
+                case "$sort_mode" in
+                    date) k="${epochs[id]:-0}" ;;
+                    size) k="${sizekb[id]:-0}" ;;
+                    name | *) k="${items[id]}|${id}" ;;
+                esac
+                printf "%s\t%s\n" "$k" "$id" >> "$tmpfile"
+            done
+
+            sorted_indices_cache=()
+            while IFS=$'\t' read -r _key _id; do
+                [[ -z "$_id" ]] && continue
+                sorted_indices_cache+=("$_id")
+            done < <(LC_ALL=C sort -t $'\t' $sort_key -- "$tmpfile" 2> /dev/null)
+
+            rm -f "$tmpfile"
+        else
+            sorted_indices_cache=("${orig_indices[@]}")
+        fi
+        sort_cache_key="$requested_key"
+    }
+
+    # Rebuild the view_indices applying filter over cached sort order
+    rebuild_view() {
+        ensure_sorted_indices
+
+        if [[ -n "$filter_text_lower" ]]; then
+            local -a source_indices=()
+            if [[ "$filter_cache_key" == "$sort_cache_key" &&
+                "$filter_text_lower" == "$filter_cache_text_lower"* &&
+                ${#filter_cache_indices[@]} -gt 0 ]]; then
+                source_indices=("${filter_cache_indices[@]}")
             else
-                # Name: ascending by default (A to Z)
-                sort_key="-k1,1f"
-                [[ "$sort_reverse" == "true" ]] && sort_key="-k1,1fr"
+                if [[ ${#sorted_indices_cache[@]} -gt 0 ]]; then
+                    source_indices=("${sorted_indices_cache[@]}")
+                else
+                    source_indices=()
+                fi
             fi
 
-            # Create temporary file for sorting
-            local tmpfile
-            tmpfile=$(mktemp 2> /dev/null) || tmpfile=""
-            if [[ -n "$tmpfile" ]]; then
-                local k id
-                for id in "${orig_indices[@]}"; do
-                    case "$sort_mode" in
-                        date) k="${epochs[id]:-0}" ;;
-                        size) k="${sizekb[id]:-0}" ;;
-                        name | *) k="${items[id]}|${id}" ;;
-                    esac
-                    printf "%s\t%s\n" "$k" "$id" >> "$tmpfile"
-                done
+            view_indices=()
+            local id
+            for id in "${source_indices[@]}"; do
+                if [[ "${filter_targets_lower[id]:-}" == *"$filter_text_lower"* ]]; then
+                    view_indices+=("$id")
+                fi
+            done
 
-                view_indices=()
-                while IFS=$'\t' read -r _key _id; do
-                    [[ -z "$_id" ]] && continue
-                    view_indices+=("$_id")
-                done < <(LC_ALL=C sort -t $'\t' $sort_key -- "$tmpfile" 2> /dev/null)
-
-                rm -f "$tmpfile"
+            filter_cache_key="$sort_cache_key"
+            filter_cache_text_lower="$filter_text_lower"
+            if [[ ${#view_indices[@]} -gt 0 ]]; then
+                filter_cache_indices=("${view_indices[@]}")
             else
-                # Fallback: no sorting
-                view_indices=("${orig_indices[@]}")
+                filter_cache_indices=()
+            fi
+        else
+            if [[ ${#sorted_indices_cache[@]} -gt 0 ]]; then
+                view_indices=("${sorted_indices_cache[@]}")
+            else
+                view_indices=()
+            fi
+            filter_cache_key="$sort_cache_key"
+            filter_cache_text_lower=""
+            if [[ ${#view_indices[@]} -gt 0 ]]; then
+                filter_cache_indices=("${view_indices[@]}")
+            else
+                filter_cache_indices=()
             fi
         fi
 
@@ -321,19 +397,45 @@ paginated_multi_select() {
         fi
     }
 
+    draw_header() {
+        printf "\033[1;1H" >&2
+        if [[ -n "$filter_text" ]]; then
+            printf "\r\033[2K${PURPLE_BOLD}%s${NC}  ${YELLOW}/ Filter: ${filter_text}_${NC}  ${GRAY}(%d/%d)${NC}\n" "${title}" "${#view_indices[@]}" "$total_items" >&2
+        elif [[ -n "${MOLE_READ_KEY_FORCE_CHAR:-}" ]]; then
+            printf "\r\033[2K${PURPLE_BOLD}%s${NC}  ${YELLOW}/ Filter: _ ${NC}${GRAY}(type to search)${NC}\n" "${title}" >&2
+        else
+            printf "\r\033[2K${PURPLE_BOLD}%s${NC}  ${GRAY}%d/%d selected${NC}\n" "${title}" "$selected_count" "$total_items" >&2
+        fi
+    }
+
+    # Handle filter character input (reduces code duplication)
+    # Returns 0 if character was handled, 1 if not in filter mode
+    handle_filter_char() {
+        local char="$1"
+        if [[ -z "${MOLE_READ_KEY_FORCE_CHAR:-}" ]]; then
+            return 1
+        fi
+        if [[ "$char" =~ ^[[:print:]]$ ]]; then
+            local char_lower
+            char_lower=$(printf "%s" "$char" | LC_ALL=C tr '[:upper:]' '[:lower:]')
+            filter_text+="$char"
+            filter_text_lower+="$char_lower"
+            rebuild_view
+            cursor_pos=0
+            top_index=0
+            need_full_redraw=true
+        fi
+        return 0
+    }
+
     # Draw the complete menu
     draw_menu() {
-        # Recalculate items_per_page dynamically to handle window resize
         items_per_page=$(_pm_calculate_items_per_page)
+        local clear_line=$'\r\033[2K'
 
         printf "\033[H" >&2
-        local clear_line="\r\033[2K"
 
-        # Use cached selection count (maintained incrementally on toggle)
-        # No need to loop through all items anymore!
-
-        # Header only
-        printf "${clear_line}${PURPLE_BOLD}%s${NC}  ${GRAY}%d/%d selected${NC}\n" "${title}" "$selected_count" "$total_items" >&2
+        draw_header
 
         # Visible slice
         local visible_total=${#view_indices[@]}
@@ -407,18 +509,21 @@ paginated_multi_select() {
         local reverse_arrow="↑"
         [[ "$sort_reverse" == "true" ]] && reverse_arrow="↓"
 
-        local refresh="${GRAY}R Refresh${NC}"
         local sort_ctrl="${GRAY}S ${sort_status}${NC}"
         local order_ctrl="${GRAY}O ${reverse_arrow}${NC}"
+        local filter_ctrl="${GRAY}/ Filter${NC}"
 
-        if [[ "$has_metadata" == "true" ]]; then
+        if [[ -n "$filter_text" ]]; then
+            local -a _segs_filter=("${GRAY}Backspace${NC}" "${GRAY}Ctrl+U Clear${NC}" "${GRAY}ESC Clear${NC}")
+            _print_wrapped_controls "$sep" "${_segs_filter[@]}"
+        elif [[ "$has_metadata" == "true" ]]; then
             # With metadata: show sort controls
             local term_width="${COLUMNS:-}"
             [[ -z "$term_width" ]] && term_width=$(tput cols 2> /dev/null || echo 80)
             [[ "$term_width" =~ ^[0-9]+$ ]] || term_width=80
 
             # Full controls
-            local -a _segs=("$nav" "$space_select" "$enter" "$refresh" "$sort_ctrl" "$order_ctrl" "$exit")
+            local -a _segs=("$nav" "$space_select" "$enter" "$sort_ctrl" "$order_ctrl" "$filter_ctrl" "$exit")
 
             # Calculate width
             local total_len=0 seg_count=${#_segs[@]}
@@ -429,7 +534,7 @@ paginated_multi_select() {
 
             # Level 1: Remove "Space Select" if too wide
             if [[ $total_len -gt $term_width ]]; then
-                _segs=("$nav" "$enter" "$refresh" "$sort_ctrl" "$order_ctrl" "$exit")
+                _segs=("$nav" "$enter" "$sort_ctrl" "$order_ctrl" "$filter_ctrl" "$exit")
 
                 total_len=0
                 seg_count=${#_segs[@]}
@@ -440,14 +545,14 @@ paginated_multi_select() {
 
                 # Level 2: Remove sort label if still too wide
                 if [[ $total_len -gt $term_width ]]; then
-                    _segs=("$nav" "$enter" "$refresh" "$order_ctrl" "$exit")
+                    _segs=("$nav" "$enter" "$order_ctrl" "$filter_ctrl" "$exit")
                 fi
             fi
 
             _print_wrapped_controls "$sep" "${_segs[@]}"
         else
             # Without metadata: basic controls
-            local -a _segs_simple=("$nav" "$space_select" "$enter" "$refresh" "$exit")
+            local -a _segs_simple=("$nav" "$space_select" "$enter" "$filter_ctrl" "$exit")
             _print_wrapped_controls "$sep" "${_segs_simple[@]}"
         fi
         printf "${clear_line}" >&2
@@ -473,52 +578,63 @@ paginated_multi_select() {
 
         case "$key" in
             "QUIT")
-                cleanup
-                return 1
+                if [[ -n "$filter_text" || -n "${MOLE_READ_KEY_FORCE_CHAR:-}" ]]; then
+                    filter_text=""
+                    filter_text_lower=""
+                    unset MOLE_READ_KEY_FORCE_CHAR
+                    rebuild_view
+                    cursor_pos=0
+                    top_index=0
+                    need_full_redraw=true
+                else
+                    cleanup
+                    return 1
+                fi
                 ;;
             "UP")
                 if [[ ${#view_indices[@]} -eq 0 ]]; then
                     :
                 elif [[ $cursor_pos -gt 0 ]]; then
-                    # Simple cursor move - only redraw affected rows
                     local old_cursor=$cursor_pos
                     ((cursor_pos--))
                     local new_cursor=$cursor_pos
 
-                    # Calculate terminal row positions (+3: row 1=header, row 2=blank, row 3=first item)
+                    if [[ -n "$filter_text" || -n "${MOLE_READ_KEY_FORCE_CHAR:-}" ]]; then
+                        draw_header
+                    fi
+
                     local old_row=$((old_cursor + 3))
                     local new_row=$((new_cursor + 3))
 
-                    # Quick redraw: update only the two affected rows
                     printf "\033[%d;1H" "$old_row" >&2
                     render_item "$old_cursor" false
                     printf "\033[%d;1H" "$new_row" >&2
                     render_item "$new_cursor" true
 
-                    # CRITICAL: Move cursor to footer to avoid visual artifacts
                     printf "\033[%d;1H" "$((items_per_page + 4))" >&2
 
                     prev_cursor_pos=$cursor_pos
-                    continue # Skip full redraw
+                    continue
                 elif [[ $top_index -gt 0 ]]; then
-                    # Scroll up - redraw visible items only
                     ((top_index--))
 
-                    # Redraw all visible items (faster than full screen redraw)
+                    if [[ -n "$filter_text" || -n "${MOLE_READ_KEY_FORCE_CHAR:-}" ]]; then
+                        draw_header
+                    fi
+
                     local start_idx=$top_index
                     local end_idx=$((top_index + items_per_page - 1))
                     local visible_total=${#view_indices[@]}
                     [[ $end_idx -ge $visible_total ]] && end_idx=$((visible_total - 1))
 
                     for ((i = start_idx; i <= end_idx; i++)); do
-                        local row=$((i - start_idx + 3)) # +3 for header
+                        local row=$((i - start_idx + 3))
                         printf "\033[%d;1H" "$row" >&2
                         local is_current=false
                         [[ $((i - start_idx)) -eq $cursor_pos ]] && is_current=true
                         render_item $((i - start_idx)) $is_current
                     done
 
-                    # Move cursor to footer
                     printf "\033[%d;1H" "$((items_per_page + 4))" >&2
 
                     prev_cursor_pos=$cursor_pos
@@ -537,28 +653,27 @@ paginated_multi_select() {
                         [[ $visible_count -gt $items_per_page ]] && visible_count=$items_per_page
 
                         if [[ $cursor_pos -lt $((visible_count - 1)) ]]; then
-                            # Simple cursor move - only redraw affected rows
                             local old_cursor=$cursor_pos
                             ((cursor_pos++))
                             local new_cursor=$cursor_pos
 
-                            # Calculate terminal row positions (+3: row 1=header, row 2=blank, row 3=first item)
+                            if [[ -n "$filter_text" || -n "${MOLE_READ_KEY_FORCE_CHAR:-}" ]]; then
+                                draw_header
+                            fi
+
                             local old_row=$((old_cursor + 3))
                             local new_row=$((new_cursor + 3))
 
-                            # Quick redraw: update only the two affected rows
                             printf "\033[%d;1H" "$old_row" >&2
                             render_item "$old_cursor" false
                             printf "\033[%d;1H" "$new_row" >&2
                             render_item "$new_cursor" true
 
-                            # CRITICAL: Move cursor to footer to avoid visual artifacts
                             printf "\033[%d;1H" "$((items_per_page + 4))" >&2
 
                             prev_cursor_pos=$cursor_pos
-                            continue # Skip full redraw
+                            continue
                         elif [[ $((top_index + visible_count)) -lt ${#view_indices[@]} ]]; then
-                            # Scroll down - redraw visible items only
                             ((top_index++))
                             visible_count=$((${#view_indices[@]} - top_index))
                             [[ $visible_count -gt $items_per_page ]] && visible_count=$items_per_page
@@ -566,21 +681,23 @@ paginated_multi_select() {
                                 cursor_pos=$((visible_count - 1))
                             fi
 
-                            # Redraw all visible items (faster than full screen redraw)
+                            if [[ -n "$filter_text" || -n "${MOLE_READ_KEY_FORCE_CHAR:-}" ]]; then
+                                draw_header
+                            fi
+
                             local start_idx=$top_index
                             local end_idx=$((top_index + items_per_page - 1))
                             local visible_total=${#view_indices[@]}
                             [[ $end_idx -ge $visible_total ]] && end_idx=$((visible_total - 1))
 
                             for ((i = start_idx; i <= end_idx; i++)); do
-                                local row=$((i - start_idx + 3)) # +3 for header
+                                local row=$((i - start_idx + 3))
                                 printf "\033[%d;1H" "$row" >&2
                                 local is_current=false
                                 [[ $((i - start_idx)) -eq $cursor_pos ]] && is_current=true
                                 render_item $((i - start_idx)) $is_current
                             done
 
-                            # Move cursor to footer
                             printf "\033[%d;1H" "$((items_per_page + 4))" >&2
 
                             prev_cursor_pos=$cursor_pos
@@ -617,21 +734,10 @@ paginated_multi_select() {
                     continue # Skip full redraw
                 fi
                 ;;
-            "RETRY")
-                # 'R' toggles reverse order (only if metadata available)
-                if [[ "$has_metadata" == "true" ]]; then
-                    if [[ "$sort_reverse" == "true" ]]; then
-                        sort_reverse="false"
-                    else
-                        sort_reverse="true"
-                    fi
-                    rebuild_view
-                    need_full_redraw=true
-                fi
-                ;;
             "CHAR:s" | "CHAR:S")
-                if [[ "$has_metadata" == "true" ]]; then
-                    # Cycle sort mode (only if metadata available)
+                if handle_filter_char "${key#CHAR:}"; then
+                    : # Handled as filter input
+                elif [[ "$has_metadata" == "true" ]]; then
                     case "$sort_mode" in
                         date) sort_mode="name" ;;
                         name) sort_mode="size" ;;
@@ -642,8 +748,9 @@ paginated_multi_select() {
                 fi
                 ;;
             "CHAR:j")
-                # Down navigation (vim style)
-                if [[ ${#view_indices[@]} -gt 0 ]]; then
+                if handle_filter_char "${key#CHAR:}"; then
+                    : # Handled as filter input
+                elif [[ ${#view_indices[@]} -gt 0 ]]; then
                     local absolute_index=$((top_index + cursor_pos))
                     local last_index=$((${#view_indices[@]} - 1))
                     if [[ $absolute_index -lt $last_index ]]; then
@@ -659,8 +766,9 @@ paginated_multi_select() {
                 fi
                 ;;
             "CHAR:k")
-                # Up navigation (vim style)
-                if [[ ${#view_indices[@]} -gt 0 ]]; then
+                if handle_filter_char "${key#CHAR:}"; then
+                    : # Handled as filter input
+                elif [[ ${#view_indices[@]} -gt 0 ]]; then
                     if [[ $cursor_pos -gt 0 ]]; then
                         ((cursor_pos--))
                         need_full_redraw=true
@@ -670,14 +778,10 @@ paginated_multi_select() {
                     fi
                 fi
                 ;;
-            "CHAR:r" | "CHAR:R")
-                # Trigger Refresh signal
-                cleanup
-                return 10
-                ;;
             "CHAR:o" | "CHAR:O")
-                if [[ "$has_metadata" == "true" ]]; then
-                    # O toggles reverse order
+                if handle_filter_char "${key#CHAR:}"; then
+                    : # Handled as filter input
+                elif [[ "$has_metadata" == "true" ]]; then
                     if [[ "$sort_reverse" == "true" ]]; then
                         sort_reverse="false"
                     else
@@ -687,6 +791,41 @@ paginated_multi_select() {
                     need_full_redraw=true
                 fi
                 ;;
+            "CHAR:/" | "CHAR:?")
+                if [[ -n "${MOLE_READ_KEY_FORCE_CHAR:-}" ]]; then
+                    unset MOLE_READ_KEY_FORCE_CHAR
+                else
+                    export MOLE_READ_KEY_FORCE_CHAR=1
+                fi
+                need_full_redraw=true
+                ;;
+            "DELETE")
+                if [[ -n "$filter_text" ]]; then
+                    filter_text="${filter_text%?}"
+                    filter_text_lower="${filter_text_lower%?}"
+                    if [[ -z "$filter_text" ]]; then
+                        filter_text_lower=""
+                        unset MOLE_READ_KEY_FORCE_CHAR
+                    fi
+                    rebuild_view
+                    cursor_pos=0
+                    top_index=0
+                    need_full_redraw=true
+                fi
+                ;;
+            "CLEAR_LINE")
+                if [[ -n "$filter_text" ]]; then
+                    filter_text=""
+                    filter_text_lower=""
+                    rebuild_view
+                    cursor_pos=0
+                    top_index=0
+                    need_full_redraw=true
+                fi
+                ;;
+            "CHAR:"*)
+                handle_filter_char "${key#CHAR:}" || true
+                ;;
             "ENTER")
                 # Smart Enter behavior
                 # 1. Check if any items are already selected
@@ -724,8 +863,9 @@ paginated_multi_select() {
 
                 trap - EXIT INT TERM
                 MOLE_SELECTION_RESULT="$final_result"
-                export MOLE_MENU_SORT_MODE="$sort_mode"
-                export MOLE_MENU_SORT_REVERSE="$sort_reverse"
+                unset MOLE_READ_KEY_FORCE_CHAR
+                export MOLE_MENU_SORT_MODE="${sort_mode:-name}"
+                export MOLE_MENU_SORT_REVERSE="${sort_reverse:-false}"
                 restore_terminal
                 return 0
                 ;;
